@@ -1,7 +1,8 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* bus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -56,6 +57,15 @@ struct _BusRegistry {
 
     GHashTable *engine_table;
     GList *active_engines;
+
+
+#ifdef G_THREADS_ENABLED
+    GThread *thread;
+    gboolean thread_running;
+    GMutex  *mutex;
+    GCond   *cond;
+    gboolean changed;
+#endif
 };
 
 struct _BusRegistryClass {
@@ -71,7 +81,7 @@ GList           *bus_registry_get_engines       (BusRegistry    *registry);
 GList           *bus_registry_get_engines_by_language
                                                 (BusRegistry    *registry,
                                                  const gchar    *language);
-void             bus_registry_stop_all_components    
+void             bus_registry_stop_all_components
                                                 (BusRegistry    *registry);
 
 IBusComponent   *bus_registry_lookup_component_by_name
@@ -84,6 +94,11 @@ BusFactoryProxy *bus_registry_name_owner_changed(BusRegistry    *registry,
                                                  const gchar    *name,
                                                  const gchar    *old_name,
                                                  const gchar    *new_name);
+#ifdef G_THREADS_ENABLED
+void             bus_registry_start_monitor_changes
+                                                (BusRegistry    *registry);
+gboolean         bus_registry_is_changed        (BusRegistry    *registry);
+#endif
 
 G_END_DECLS
 #endif
