@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright (c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright (c) 2007-2009 Red Hat, Inc.
+# Copyright (c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2010 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -49,16 +49,25 @@ class Bus(object.Object):
             gobject.TYPE_NONE,
             ()
         ),
+        "registry-changed" : (
+            gobject.SIGNAL_RUN_LAST,
+            gobject.TYPE_NONE,
+            ()
+        ),
     }
 
     def __init__(self):
         super(Bus, self).__init__()
         self.__dbusconn = dbus.connection.Connection(common.get_address())
-        self.__dbus = self.__dbusconn.get_object(dbus.BUS_DAEMON_NAME,
-                                                 dbus.BUS_DAEMON_PATH)
+        _dbus = self.__dbusconn.get_object(dbus.BUS_DAEMON_NAME,
+                                           dbus.BUS_DAEMON_PATH)
+        self.__dbus = dbus.Interface (_dbus, dbus_interface="org.freedesktop.DBus")
         self.__unique_name = self.hello()
-        self.__ibus = self.__dbusconn.get_object(common.IBUS_SERVICE_IBUS,
-                                                 common.IBUS_PATH_IBUS)
+
+        _ibus = self.__dbusconn.get_object(common.IBUS_SERVICE_IBUS,
+                                           common.IBUS_PATH_IBUS)
+        self.__ibus = dbus.Interface (_ibus, dbus_interface='org.freedesktop.IBus')
+        self.__ibus.connect_to_signal("RegistryChanged", self.__registry_changed_cb)
 
         self.__dbusconn.call_on_disconnection(self.__dbusconn_disconnected_cb)
         # self.__dbusconn.add_message_filter(self.__filter_cb)
@@ -75,6 +84,9 @@ class Bus(object.Object):
         self.__dbusconn = None
         self.emit("disconnected")
 
+    def __registry_changed_cb(self):
+        self.emit("registry-changed")
+
     def get_name(self):
         return self.__unique_name
 
