@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright (c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright (c) 2007-2009 Red Hat, Inc.
+# Copyright (c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2010 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,8 @@ __all__ = ("IIBus", )
 
 import dbus.service
 from ibus.common import \
-    IBUS_IFACE_IBUS
+    IBUS_IFACE_IBUS, \
+    IBUS_IFACE_CONFIG
 
 class IIBus(dbus.service.Object):
     # define method decorator.
@@ -40,6 +41,11 @@ class IIBus(dbus.service.Object):
                             async_callbacks=("reply_cb", "error_cb"), \
                             **args)
 
+    # define signal decorator.
+    signal = lambda **args: \
+        dbus.service.signal(dbus_interface=IBUS_IFACE_IBUS, \
+            **args)
+
     @method(out_signature="s")
     def GetAddress(self, dbusconn): pass
 
@@ -47,7 +53,7 @@ class IIBus(dbus.service.Object):
     @method(in_signature="s", out_signature="o")
     def CreateInputContext(self, client_name, dbusconn): pass
 
-    @method(out_signature="s")
+    @method(out_signature="o")
     def CurrentInputContext(self, dbusconn): pass
 
     # general methods
@@ -66,3 +72,6 @@ class IIBus(dbus.service.Object):
     @method(in_signature="v", out_signature="v")
     def Ping(self, data, dbusconn): pass
 
+    @signal(signature="")
+    def RegistryChanged(self): pass
+
