@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright (c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright (c) 2007-2009 Red Hat, Inc.
+# Copyright (c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2010 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -107,25 +107,25 @@ class Setup(object):
 
         # next engine
         shortcuts = self.__config.get_value(
-                        "general/hotkey", "next_engine",
+                        "general/hotkey", "next_engine_in_menu",
                         ibus.CONFIG_GENERAL_SHORTCUT_NEXT_ENGINE_DEFAULT)
         button = self.__builder.get_object("button_next_engine")
         entry = self.__builder.get_object("entry_next_engine")
         entry.set_text("; ".join(shortcuts))
         entry.set_tooltip_text("\n".join(shortcuts))
         button.connect("clicked", self.__shortcut_button_clicked_cb,
-                    N_("next input method"), "general/hotkey", "next_engine", entry)
+                    N_("next input method"), "general/hotkey", "next_engine_in_menu", entry)
 
         # prev engine
         shortcuts = self.__config.get_value(
-                        "general/hotkey", "prev_engine",
+                        "general/hotkey", "previous_engine",
                         ibus.CONFIG_GENERAL_SHORTCUT_PREV_ENGINE_DEFAULT)
         button = self.__builder.get_object("button_prev_engine")
         entry = self.__builder.get_object("entry_prev_engine")
         entry.set_text("; ".join(shortcuts))
         entry.set_tooltip_text("\n".join(shortcuts))
         button.connect("clicked", self.__shortcut_button_clicked_cb,
-                    N_("previous input method"), "general/hotkey", "prev_engine", entry)
+                    N_("previous input method"), "general/hotkey", "previous_engine", entry)
 
         # lookup table orientation
         self.__combobox_lookup_table_orientation = self.__builder.get_object("combobox_lookup_table_orientation")
@@ -137,7 +137,7 @@ class Setup(object):
         # auto hide
         self.__combobox_panel_show = self.__builder.get_object("combobox_panel_show")
         self.__combobox_panel_show.set_active(
-            self.__config.get_value("panel", "show", 1))
+            self.__config.get_value("panel", "show", 0))
         self.__combobox_panel_show.connect("changed", self.__combobox_panel_show_changed_cb)
 
         # panel position
@@ -175,35 +175,40 @@ class Setup(object):
             self.__config.get_value("panel", "show_im_name", False))
         self.__checkbutton_show_im_name.connect("toggled", self.__checkbutton_show_im_name_toggled_cb)
 
+        # embed preedit text
+        self.__checkbutton_embed_preedit_text = self.__builder.get_object("checkbutton_embed_preedit_text")
+        self.__checkbutton_embed_preedit_text.set_active(
+            self.__config.get_value("general", "embed_preedit_text", True))
+        self.__checkbutton_embed_preedit_text.connect("toggled", self.__checkbutton_embed_preedit_text_toggled_cb)
+
         # use system keyboard layout setting
         self.__checkbutton_use_sys_layout = self.__builder.get_object("checkbutton_use_sys_layout")
         self.__checkbutton_use_sys_layout.set_active(
-            self.__config.get_value("general", "use_system_keyboard_layout", False))
+            self.__config.get_value("general", "use_system_keyboard_layout", True))
         self.__checkbutton_use_sys_layout.connect("toggled", self.__checkbutton_use_sys_layout_toggled_cb)
 
+        # use global ime setting
+        self.__checkbutton_use_global_engine = self.__builder.get_object("checkbutton_use_global_engine")
+        self.__checkbutton_use_global_engine.set_active(
+            self.__config.get_value("general", "use_global_engine", False))
+        self.__checkbutton_use_global_engine.connect("toggled", self.__checkbutton_use_global_engine_toggled_cb)
+
         # init engine page
         self.__engines = self.__bus.list_engines()
-        self.__combobox = EngineComboBox(self.__engines)
-        self.__combobox.show()
-        self.__builder.get_object("alignment_engine_combobox").add(self.__combobox)
+        self.__combobox = self.__builder.get_object("combobox_engines")
+        self.__combobox.set_engines(self.__engines)
 
         tmp_dict = {}
         for e in self.__engines:
             tmp_dict[e.name] = e
         engine_names = self.__config.get_value("general", "preload_engines", [])
-        engines = []
-        for n in engine_names:
-            if n in tmp_dict:
-                engines.append(tmp_dict[n])
-        self.__treeview = EngineTreeView(engines)
-        self.__treeview.show()
-        self.__builder.get_object("scrolledwindow_engine_treeview").add(self.__treeview)
+        engines = [tmp_dict[name] for name in engine_names if name in tmp_dict]
 
-        self.__treeview.connect("changed", self.__treeview_changed_cb)
+        self.__treeview = self.__builder.get_object("treeview_engines")
+        self.__treeview.set_engines(engines)
 
         button = self.__builder.get_object("button_engine_add")
-        button.connect("clicked",
-                       lambda *args:self.__treeview.append_engine(self.__combobox.get_active_engine()))
+        button.connect("clicked", self.__button_engine_add_cb)
         button = self.__builder.get_object("button_engine_remove")
         button.connect("clicked", lambda *args:self.__treeview.remove_engine())
         button = self.__builder.get_object("button_engine_up")
@@ -215,18 +220,41 @@ class Setup(object):
         button = self.__builder.get_object("button_engine_about")
         button.connect("clicked", self.__button_engine_about_cb)
 
+        self.__combobox.connect("notify::active-engine", self.__combobox_notify_active_engine_cb)
+        self.__treeview.connect("notify", self.__treeview_notify_cb)
+
+    def __combobox_notify_active_engine_cb(self, combobox, property):
+        engine = self.__combobox.get_active_engine()
+        button = self.__builder.get_object("button_engine_add")
+        button.set_sensitive(engine != None and engine not in self.__treeview.get_engines())
+
+    def __treeview_notify_cb(self, treeview, property):
+        if property.name != "active-engine" and property.name != "engines":
+            return
+
+        engines = self.__treeview.get_engines()
+        engine = self.__treeview.get_active_engine()
+
+        self.__builder.get_object("button_engine_remove").set_sensitive(engine != None)
+        self.__builder.get_object("button_engine_about").set_sensitive(engine != None)
+        self.__builder.get_object("button_engine_up").set_sensitive(engine not in engines[:1])
+        self.__builder.get_object("button_engine_down").set_sensitive(engine not in engines[-1:])
+
+        if property.name == "engines":
+            engine_names = map(lambda e: e.name, engines)
+            self.__config.set_list("general", "preload_engines", engine_names, "s")
+
+    def __button_engine_add_cb(self, button):
+        engine = self.__combobox.get_active_engine()
+        self.__treeview.append_engine(engine)
+
     def __button_engine_about_cb(self, button):
-        engine = self.__treeview.get_select_engine()
+        engine = self.__treeview.get_active_engine()
         if engine:
             about = EngineAbout(engine)
             about.run()
             about.destroy()
 
-    def __treeview_changed_cb(self, treeview):
-        engines = self.__treeview.get_engines()
-        engine_names = map(lambda e: e.name, engines)
-        self.__config.set_list("general", "preload_engines", engine_names, "s")
-
     def __init_bus(self):
         try:
             self.__bus = ibus.Bus()
@@ -246,7 +274,7 @@ class Setup(object):
                 self.__flush_gtk_events()
                 if id != gtk.RESPONSE_YES:
                     sys.exit(0)
-                pid = os.spawnlp(os.P_NOWAIT, "ibus-daemon", "ibus-daemon")
+                pid = os.spawnlp(os.P_NOWAIT, "ibus-daemon", "ibus-daemon", "--xim")
                 time.sleep(1)
                 try:
                     self.__bus = ibus.Bus()
@@ -404,10 +432,18 @@ class Setup(object):
         value = self.__checkbutton_show_im_name.get_active()
         self.__config.set_value("panel", "show_im_name", value)
 
+    def __checkbutton_embed_preedit_text_toggled_cb(self, button):
+        value = self.__checkbutton_embed_preedit_text.get_active()
+        self.__config.set_value("general", "embed_preedit_text", value)
+
     def __checkbutton_use_sys_layout_toggled_cb(self, button):
         value = self.__checkbutton_use_sys_layout.get_active()
         self.__config.set_value("general", "use_system_keyboard_layout", value)
 
+    def __checkbutton_use_global_engine_toggled_cb(self, button):
+        value = self.__checkbutton_use_global_engine.get_active()
+        self.__config.set_value("general", "use_global_engine", value)
+
     def __config_value_changed_cb(self, bus, section, name, value):
         pass
 
