@@ -1,7 +1,8 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -28,12 +29,10 @@
 #ifndef __IBUS_BUS_H_
 #define __IBUS_BUS_H_
 
-#include <dbus/dbus.h>
 #include "ibusinputcontext.h"
 #include "ibusconfig.h"
 #include "ibuscomponent.h"
 
-
 /*
  * Type macros.
  */
@@ -75,7 +74,7 @@ struct _IBusBusClass {
 GType        ibus_bus_get_type          (void);
 /**
  * ibus_bus_new:
- * @returns: A newly allocated IBusBus instance.
+ * @returns: A newly allocated IBusBus instance, and the instance is not floating.
  *
  * New an IBusBus instance.
  */
@@ -99,7 +98,15 @@ gboolean     ibus_bus_is_connected      (IBusBus        *bus);
  */
 IBusConnection
             *ibus_bus_get_connection    (IBusBus        *bus);
-/* declare dbus methods */
+
+/**
+ * ibus_bus_hello:
+ * @bus: An IBusBus.
+ * @returns: The unique name of IBus process in DBus.
+ *
+ * This function sends a "HELLO" message to DBus daemon,
+ * which replies the unique name of current IBus process.
+ */
 const gchar *ibus_bus_hello             (IBusBus        *bus);
 
 /**
@@ -140,7 +147,7 @@ gboolean     ibus_bus_name_has_owner    (IBusBus        *bus,
 /**
  * ibus_bus_list_names:
  * @bus: An IBusBus.
- * @returns: Lists that attached to @bus.
+ * @returns: (transfer full) (element-type utf8): Lists that attached to @bus.
  *
  * Return lists that attached to @bus.
  * <note><para>[FixMe] Not implemented yet, only return NULL.</para></note>
@@ -171,11 +178,11 @@ void         ibus_bus_remove_match      (IBusBus        *bus,
  * ibus_bus_get_name_owner:
  * @bus: An IBusBus.
  * @name: Name.
- * @returns: Owner of the name.
+ * @returns: Owner of the name. The returned value must be freed with g_free().
  *
  * Return the name owner.
  */
-const gchar *ibus_bus_get_name_owner    (IBusBus        *bus,
+gchar       *ibus_bus_get_name_owner    (IBusBus        *bus,
                                          const gchar    *name);
 /* declare ibus methods */
 
@@ -208,12 +215,13 @@ IBusInputContext
 /**
  * ibus_bus_current_input_context:
  * @bus: An IBusBus.
- * @returns: The named of currently focued IBusInputContext if the "CurrentInputContext" call
- *            suceeded, NULL otherwise.
+ * @returns: The named of currently focued IBusInputContext if the
+ *            "CurrentInputContext" call suceeded, NULL otherwise. The return
+ *            value must be freed with g_free().
  *
  * Get the current focused input context.
  */
-const gchar *ibus_bus_current_input_context(IBusBus        *bus);
+gchar       *ibus_bus_current_input_context(IBusBus        *bus);
 
 
 /**
@@ -230,7 +238,7 @@ gboolean     ibus_bus_register_component(IBusBus        *bus,
 /**
  * ibus_bus_list_engines:
  * @bus: An IBusBus.
- * @returns: A GList of engines.
+ * @returns: (transfer full) (element-type IBusEngineDesc): A List of engines.
  *
  * List engines.
  * Note that this function is not yet implemented.
@@ -240,7 +248,7 @@ GList       *ibus_bus_list_engines      (IBusBus        *bus);
 /**
  * ibus_bus_list_active_engines:
  * @bus: An IBusBus.
- * @returns: A GList of active engines.
+ * @returns: (transfer full) (element-type IBusEngineDesc): A List of active engines.
  *
  * List active engines.
  * Note that this function is not yet implemented.
@@ -248,6 +256,59 @@ GList       *ibus_bus_list_engines      (IBusBus        *bus);
  */
 GList       *ibus_bus_list_active_engines
                                         (IBusBus        *bus);
+
+/**
+ * ibus_bus_get_use_sys_layout:
+ * @bus: An IBusBus.
+ * @returns: TRUE if "use_sys_layout" option is enabled.
+ *
+ * Check if the bus's "use_sys_layout" option is enabled or not.
+ */
+gboolean     ibus_bus_get_use_sys_layout(IBusBus        *bus);
+
+/**
+ * ibus_bus_get_use_global_engine:
+ * @bus: An IBusBus.
+ * @returns: TRUE if "use_global_engine" option is enabled.
+ *
+ * Check if the bus's "use_global_engine" option is enabled or not.
+ */
+gboolean     ibus_bus_get_use_global_engine
+                                        (IBusBus        *bus);
+
+
+/**
+ * ibus_bus_is_global_engine_enabled:
+ * @bus: An IBusBus.
+ * @returns: TRUE if the current global engine is enabled.
+ *
+ * Check if the current global engine is enabled or not.
+ */
+gboolean     ibus_bus_is_global_engine_enabled
+                                        (IBusBus        *bus);
+
+/**
+ * ibus_bus_get_global_engine:
+ * @bus: An IBusBus.
+ * @returns: The description of current global engine, or NULL if there is no
+ * global engine.
+ *
+ * Get the description of current global engine.
+ */
+IBusEngineDesc
+            *ibus_bus_get_global_engine (IBusBus        *bus);
+
+/**
+ * ibus_bus_set_global_engine:
+ * @bus: An IBusBus.
+ * @global_engine: A new engine name.
+ * @returns: TRUE if the global engine was set successfully.
+ *
+ * Set current global engine.
+ */
+gboolean     ibus_bus_set_global_engine (IBusBus        *bus,
+                                         const gchar    *global_engine);
+
 /* declare config apis */
 /**
  * ibus_bus_get_config:
@@ -260,4 +321,3 @@ GList       *ibus_bus_list_active_engines
 
 G_END_DECLS
 #endif
-
