@@ -1,7 +1,8 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +21,7 @@
  */
 /**
  * SECTION: ibusconfig
+ * @title: IBusConfig
  * @short_description: IBus engine configuration module.
  *
  * An IBusConfig provides engine configuration methods
@@ -120,6 +122,20 @@ gboolean         ibus_config_set_value      (IBusConfig         *config,
                                              const gchar        *section,
                                              const gchar        *name,
                                              const GValue       *value);
+
+/**
+ * ibus_config_unset:
+ * @config: An IBusConfig
+ * @section: Section name of the configuration option.
+ * @name: Name of the configure option its self.
+ * @returns: TRUE if succeed; FALSE otherwise.
+ *
+ * Remove an entry of a configuration option.
+ * @see_also: ibus_config_get_value.
+ */
+gboolean         ibus_config_unset      (IBusConfig         *config,
+                                         const gchar        *section,
+                                         const gchar        *name);
 G_END_DECLS
 #endif
 
