@@ -1,3 +1,4 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (c) 2009, Google Inc. All rights reserved.
@@ -17,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
+#include <dbus/dbus.h>
 #include "ibusshare.h"
 #include "ibuspanelservice.h"
 
@@ -27,8 +28,6 @@ enum {
 };
 
 /* functions prototype */
-static void     ibus_panel_service_class_init            (IBusPanelServiceClass *klass);
-static void     ibus_panel_service_init                  (IBusPanelService      *panel);
 static void     ibus_panel_service_service_set_property  (IBusPanelService      *panel,
                                                           guint                  prop_id,
                                                           const GValue          *value,
@@ -75,33 +74,7 @@ static gboolean ibus_panel_service_update_property       (IBusPanelService
                                                           IBusProperty          *prop,
                                                           IBusError            **error);
 
-static IBusServiceClass  *parent_class = NULL;
-
-GType
-ibus_panel_service_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (IBusPanelServiceClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_panel_service_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (IBusPanelService),
-        0,
-        (GInstanceInitFunc) ibus_panel_service_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_SERVICE,
-                                       "IBusPanelService",
-                                       &type_info,
-                                       (GTypeFlags)0);
-    }
-    return type;
-}
+G_DEFINE_TYPE (IBusPanelService, ibus_panel_service, IBUS_TYPE_SERVICE)
 
 IBusPanelService *
 ibus_panel_service_new (IBusConnection *connection)
@@ -123,8 +96,6 @@ ibus_panel_service_class_init (IBusPanelServiceClass *klass)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-    parent_class = (IBusServiceClass *) g_type_class_peek_parent (klass);
-
     gobject_class->set_property = (GObjectSetPropertyFunc) ibus_panel_service_service_set_property;
     gobject_class->get_property = (GObjectGetPropertyFunc) ibus_panel_service_service_get_property;
 
@@ -211,7 +182,6 @@ ibus_panel_service_service_get_property (IBusPanelService *panel,
             else {
                 g_value_set_object (value, NULL);
             }
-            g_list_foreach (connections, (GFunc) g_object_unref, NULL);
             g_list_free (connections);
         }
         break;
@@ -223,7 +193,7 @@ ibus_panel_service_service_get_property (IBusPanelService *panel,
 static void
 ibus_panel_service_real_destroy (IBusPanelService *panel)
 {
-    IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (panel));
+    IBUS_OBJECT_CLASS(ibus_panel_service_parent_class)->destroy (IBUS_OBJECT (panel));
 }
 
 static gboolean
@@ -255,13 +225,12 @@ ibus_panel_service_ibus_message (IBusPanelService *panel,
         { "ShowPreeditText",       G_STRUCT_OFFSET (IBusPanelServiceClass, show_preedit_text) },
         { "StartSetup",            G_STRUCT_OFFSET (IBusPanelServiceClass, start_setup) },
         { "StateChanged",          G_STRUCT_OFFSET (IBusPanelServiceClass, state_changed) },
-        { NULL, 0 },
     };
 
     IBusMessage *reply = NULL;
 
     gint i;
-    for (i = 0; no_arg_methods[i].name != NULL; i++) {
+    for (i = 0; i < G_N_ELEMENTS (no_arg_methods); i++) {
         if (!ibus_message_is_method_call (message, IBUS_INTERFACE_PANEL,
                                           no_arg_methods[i].name))
             continue;
@@ -364,6 +333,9 @@ ibus_panel_service_ibus_message (IBusPanelService *panel,
         else {
             reply = ibus_message_new_method_return (message);
         }
+
+        if (prop_list != NULL && g_object_is_floating (prop_list))
+            g_object_unref (prop_list);
     }
     else if (ibus_message_is_method_call (message, IBUS_INTERFACE_PANEL, "UpdateAuxiliaryText")) {
         IBusText *text = NULL;
@@ -389,6 +361,9 @@ ibus_panel_service_ibus_message (IBusPanelService *panel,
         else {
             reply = ibus_message_new_method_return (message);
         }
+
+        if (text != NULL && g_object_is_floating (text))
+            g_object_unref (text);
     }
     else if (ibus_message_is_method_call (message, IBUS_INTERFACE_PANEL, "UpdateLookupTable")) {
         IBusLookupTable *table = NULL;
@@ -414,6 +389,9 @@ ibus_panel_service_ibus_message (IBusPanelService *panel,
         else {
             reply = ibus_message_new_method_return (message);
         }
+
+        if (table != NULL && g_object_is_floating (table))
+            g_object_unref (table);
     }
     else if (ibus_message_is_method_call (message, IBUS_INTERFACE_PANEL, "UpdatePreeditText")) {
         IBusText *text = NULL;
@@ -442,6 +420,9 @@ ibus_panel_service_ibus_message (IBusPanelService *panel,
         else {
             reply = ibus_message_new_method_return (message);
         }
+
+        if (text != NULL && g_object_is_floating (text))
+            g_object_unref (text);
     }
     else if (ibus_message_is_method_call (message, IBUS_INTERFACE_PANEL, "UpdateProperty")) {
         IBusProperty *property = NULL;
@@ -464,6 +445,9 @@ ibus_panel_service_ibus_message (IBusPanelService *panel,
         else {
             reply = ibus_message_new_method_return (message);
         }
+
+        if (property != NULL && g_object_is_floating (property))
+            g_object_unref (property);
     }
     else if (ibus_message_is_method_call (message, IBUS_INTERFACE_PANEL, "SetCursorLocation")) {
         guint x, y, w, h;
@@ -661,9 +645,3 @@ ibus_panel_service_property_hide (IBusPanelService *panel,
                               G_TYPE_STRING, &prop_name,
                               G_TYPE_INVALID);
 }
-/* For Emacs:
- * Local Variables:
- * c-file-style: "gnu"
- * c-basic-offset: 4
- * End:
- */
