@@ -1,3 +1,4 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
  * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
@@ -39,13 +40,6 @@
 
 G_BEGIN_DECLS
 
-/**
- * IBusPendingCall:
- *
- * An opaque data structure that represents IBusPendingCall.
- */
-typedef DBusPendingCall IBusPendingCall;
-
 /**
  * IBusPendingCallNotifyFunction:
  * @pending: An IBusPendingCall.
