@@ -1,7 +1,8 @@
+/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2009 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2009 Red Hat, Inc.
+ * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2010 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,6 +21,7 @@
  */
 
 #include <stdarg.h>
+#include <dbus/dbus.h>
 #include "ibusmessage.h"
 #include "ibusproxy.h"
 #include "ibusinternal.h"
@@ -54,8 +56,6 @@ typedef struct _IBusProxyPrivate IBusProxyPrivate;
 static guint            proxy_signals[LAST_SIGNAL] = { 0 };
 
 /* functions prototype */
-static void      ibus_proxy_class_init  (IBusProxyClass *klass);
-static void      ibus_proxy_init        (IBusProxy      *proxy);
 static GObject  *ibus_proxy_constructor (GType           type,
                                          guint           n_construct_params,
                                          GObjectConstructParam *construct_params);
@@ -72,33 +72,7 @@ static void      ibus_proxy_get_property(IBusProxy       *proxy,
 static gboolean  ibus_proxy_ibus_signal (IBusProxy       *proxy,
                                          IBusMessage     *message);
 
-static IBusObjectClass  *parent_class = NULL;
-
-GType
-ibus_proxy_get_type (void)
-{
-    static GType type = 0;
-
-    static const GTypeInfo type_info = {
-        sizeof (IBusProxyClass),
-        (GBaseInitFunc)     NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc)    ibus_proxy_class_init,
-        NULL,               /* class finalize */
-        NULL,               /* class data */
-        sizeof (IBusProxy),
-        0,
-        (GInstanceInitFunc) ibus_proxy_init,
-    };
-
-    if (type == 0) {
-        type = g_type_register_static (IBUS_TYPE_OBJECT,
-                    "IBusProxy",
-                    &type_info,
-                    (GTypeFlags)0);
-    }
-    return type;
-}
+G_DEFINE_TYPE (IBusProxy, ibus_proxy, IBUS_TYPE_OBJECT)
 
 IBusProxy *
 ibus_proxy_new (const gchar     *name,
@@ -126,8 +100,6 @@ ibus_proxy_class_init (IBusProxyClass *klass)
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (klass);
 
-    parent_class = (IBusObjectClass *) g_type_class_peek_parent (klass);
-
     g_type_class_add_private (klass, sizeof (IBusProxyPrivate));
 
     gobject_class->constructor = ibus_proxy_constructor;
@@ -260,7 +232,7 @@ ibus_proxy_constructor (GType           type,
     IBusProxy *proxy;
     IBusProxyPrivate *priv;
 
-    obj = G_OBJECT_CLASS (parent_class)->constructor (type, n_construct_params, construct_params);
+    obj = G_OBJECT_CLASS (ibus_proxy_parent_class)->constructor (type, n_construct_params, construct_params);
 
     proxy = IBUS_PROXY (obj);
     priv = IBUS_PROXY_GET_PRIVATE (proxy);
@@ -294,7 +266,6 @@ ibus_proxy_constructor (GType           type,
                                    "AddMatch",
                                    &error,
                                    G_TYPE_STRING, &rule,
-                                   G_TYPE_INVALID,
                                    G_TYPE_INVALID)) {
             g_warning ("%s: %s", error->name, error->message);
             ibus_error_free (error);
@@ -314,7 +285,6 @@ ibus_proxy_constructor (GType           type,
                                    "AddMatch",
                                    &error,
                                    G_TYPE_STRING, &rule,
-                                   G_TYPE_INVALID,
                                    G_TYPE_INVALID)) {
             g_warning ("%s: %s", error->name, error->message);
             ibus_error_free (error);
@@ -353,7 +323,7 @@ ibus_proxy_destroy (IBusProxy *proxy)
     priv = IBUS_PROXY_GET_PRIVATE (proxy);
 
     if (priv->connection) {
-
+        /* disconnect signal handlers */
         g_signal_handlers_disconnect_by_func (priv->connection,
                                               (GCallback) _connection_ibus_signal_cb,
                                               proxy);
@@ -361,7 +331,8 @@ ibus_proxy_destroy (IBusProxy *proxy)
                                               (GCallback) _connection_destroy_cb,
                                               proxy);
 
-        if (priv->name != NULL) {
+        /* remove match rules */
+        if (priv->name != NULL && ibus_connection_is_connected (priv->connection)) {
 
             IBusError *error;
             gchar *rule;
@@ -381,7 +352,6 @@ ibus_proxy_destroy (IBusProxy *proxy)
                                        "RemoveMatch",
                                        &error,
                                        G_TYPE_STRING, &rule,
-                                       G_TYPE_INVALID,
                                        G_TYPE_INVALID)) {
 
                 g_warning ("%s: %s", error->name, error->message);
@@ -402,7 +372,6 @@ ibus_proxy_destroy (IBusProxy *proxy)
                                        "RemoveMatch",
                                        &error,
                                        G_TYPE_STRING, &rule,
-                                       G_TYPE_INVALID,
                                        G_TYPE_INVALID)) {
 
                 g_warning ("%s: %s", error->name, error->message);
@@ -427,7 +396,7 @@ ibus_proxy_destroy (IBusProxy *proxy)
         priv->connection = NULL;
     }
 
-    IBUS_OBJECT_CLASS(parent_class)->destroy (IBUS_OBJECT (proxy));
+    IBUS_OBJECT_CLASS(ibus_proxy_parent_class)->destroy (IBUS_OBJECT (proxy));
 }
 
 static void
@@ -455,7 +424,7 @@ ibus_proxy_set_property (IBusProxy      *proxy,
     case PROP_CONNECTION:
         g_assert (priv->connection == NULL);
         priv->connection = IBUS_CONNECTION (g_value_get_object (value));
-        g_object_ref (priv->connection);
+        g_object_ref_sink (priv->connection);
         break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (proxy, prop_id, pspec);
@@ -550,22 +519,28 @@ ibus_proxy_get_unique_name (IBusProxy *proxy)
     priv = IBUS_PROXY_GET_PRIVATE (proxy);
 
     if (priv->unique_name == NULL && priv->connection != NULL) {
-        IBusError *error;
-        gchar *owner;
-        if (!ibus_connection_call (priv->connection,
-                                   DBUS_SERVICE_DBUS,
-                                   DBUS_PATH_DBUS,
-                                   DBUS_INTERFACE_DBUS,
-                                   "GetNameOwner",
-                                   &error,
-                                   G_TYPE_STRING, &(priv->name),
-                                   G_TYPE_INVALID,
-                                   G_TYPE_STRING, &owner,
-                                   G_TYPE_INVALID)) {
-            g_warning ("%s: %s", error->name, error->message);
-            ibus_error_free (error);
+        IBusMessage *reply = NULL;
+        IBusError *error = NULL;
+        gchar *owner = NULL;
+        reply = ibus_connection_call_with_reply (priv->connection,
+                                                 DBUS_SERVICE_DBUS,
+                                                 DBUS_PATH_DBUS,
+                                                 DBUS_INTERFACE_DBUS,
+                                                 "GetNameOwner",
+                                                 NULL,
+                                                 G_TYPE_STRING, &(priv->name),
+                                                 G_TYPE_INVALID);
+        if (reply) {
+            if (ibus_message_get_args (reply, &error, G_TYPE_STRING, &owner,
+                                       G_TYPE_INVALID)) {
+                priv->unique_name = g_strdup (owner);
+            } else {
+                g_warning ("%s: %s", error->name, error->message);
+                ibus_error_free (error);
+            }
+
+            ibus_message_unref (reply);
         }
-        priv->unique_name = g_strdup (owner);
     }
 
     return priv->unique_name;
