@@ -2,8 +2,8 @@
 #
 # ibus - The Input Bus
 #
-# Copyright(c) 2007-2009 Peng Huang <shawn.p.huang@gmail.com>
-# Copyright(c) 2007-2009 Red Hat, Inc.
+# Copyright(c) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright(c) 2007-2010 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@
 
 import gtk
 import gtk.gdk as gdk
+import glib
 import gobject
 import ibus
 import icon as _icon
@@ -63,15 +64,12 @@ class Panel(ibus.PanelBase):
         self.__bus = bus
         self.__config = self.__bus.get_config()
         self.__focus_ic = None
-        self.__setup_pid = 0
+        self.__setup_pid = None
         self.__prefix = os.getenv("IBUS_PREFIX")
         self.__data_dir = path.join(self.__prefix, "share", "ibus")
         # self.__icons_dir = path.join(self.__data_dir, "icons")
         self.__setup_cmd = path.join(self.__prefix, "bin", "ibus-setup")
 
-        # hanlder signal
-        signal.signal(signal.SIGCHLD, self.__sigchld_cb)
-
         # connect bus signal
         self.__config.connect("value-changed", self.__config_value_changed_cb)
         self.__config.connect("reloaded", self.__config_reloaded_cb)
@@ -122,7 +120,7 @@ class Panel(ibus.PanelBase):
         # self.__bus.request_name(ibus.panel.IBUS_SERVICE_PANEL, 0)
 
     def set_cursor_location(self, x, y, w, h):
-        self.__candidate_panel.set_cursor_location(x + w, y + h)
+        self.__candidate_panel.set_cursor_location(x, y, w, h)
 
     def update_preedit_text(self, text, cursor_pos, visible):
         self.__candidate_panel.update_preedit_text(text, cursor_pos, visible)
@@ -256,19 +254,17 @@ class Panel(ibus.PanelBase):
 
     def __config_load_lookup_table_orientation(self):
         value = self.__config.get_value("panel", "lookup_table_orientation", 0)
-        if value != 0 and value != 1:
-            value = 0
-        if value == 0:
-            self.__candidate_panel.set_orientation(gtk.ORIENTATION_HORIZONTAL)
+        if value in (ibus.ORIENTATION_HORIZONTAL, ibus.ORIENTATION_VERTICAL):
+            orientation = value
         else:
-            self.__candidate_panel.set_orientation(gtk.ORIENTATION_VERTICAL)
+            orientation = ibus.ORIENTATION_HORIZONTAL
+        self.__candidate_panel.set_orientation(orientation)
 
     def __config_load_show(self):
-        show = self.__config.get_value("panel", "show", 1)
+        show = self.__config.get_value("panel", "show", 0)
         self.__language_bar.set_show(show)
 
     def __config_load_position(self):
-        position = self.__config.get_value("panel", "position", 3)
         x = self.__config.get_value("panel", "x", -1)
         y = self.__config.get_value("panel", "y", -1)
         self.__language_bar.set_position(x, y)
@@ -305,8 +301,6 @@ class Panel(ibus.PanelBase):
             self.__config_load_lookup_table_orientation()
         elif name == "show":
             self.__config_load_show()
-        # elif name == "position":
-        #     self.__config_load_position()
         elif name == "use_custom_font" or name == "custom_font":
             self.__config_load_custom_font()
         elif name == "show_icon_on_systray":
@@ -345,51 +339,85 @@ class Panel(ibus.PanelBase):
         menu.set_take_focus(False)
         return menu
 
+    # def __create_im_menu(self):
+    #     menu = gtk.Menu()
+    #     engines = self.__bus.list_active_engines()
+
+    #     tmp = {}
+    #     for engine in engines:
+    #         lang = ibus.get_language_name(engine.language)
+    #         if lang not in tmp:
+    #             tmp[lang] = []
+    #         tmp[lang].append(engine)
+
+    #     langs = tmp.keys()
+    #     other = tmp.get(_("Other"), [])
+    #     if _("Other") in tmp:
+    #         langs.remove(_("Other"))
+    #         langs.append(_("Other"))
+
+    #     size = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)
+    #     for lang in langs:
+    #         if len(tmp[lang]) == 1:
+    #             engine = tmp[lang][0]
+    #             item = gtk.ImageMenuItem("%s - %s" % (lang, engine.longname))
+    #             if engine.icon:
+    #                 item.set_image(_icon.IconWidget(engine.icon, size[0]))
+    #             else:
+    #                 item.set_image(_icon.IconWidget(ICON_ENGINE, size[0]))
+    #             item.connect("activate", self.__im_menu_item_activate_cb, engine)
+    #             menu.add(item)
+    #         else:
+    #             item = gtk.MenuItem(lang)
+    #             menu.add(item)
+    #             submenu = gtk.Menu()
+    #             item.set_submenu(submenu)
+    #             for engine in tmp[lang]:
+    #                 item = gtk.ImageMenuItem(engine.longname)
+    #                 if engine.icon:
+    #                     item.set_image(_icon.IconWidget(engine.icon, size[0]))
+    #                 else:
+    #                     item.set_image(_icon.IconWidget(ICON_ENGINE, size[0]))
+    #                 item.connect("activate", self.__im_menu_item_activate_cb, engine)
+    #                 submenu.add(item)
+
+    #     item = gtk.ImageMenuItem(_("Turn off input method"))
+    #     item.set_image(_icon.IconWidget("gtk-close", size[0]))
+    #     item.connect("activate", self.__im_menu_item_activate_cb, None)
+    #     menu.add(item)
+
+    #     menu.show_all()
+    #     menu.set_take_focus(False)
+    #     return menu
+
     def __create_im_menu(self):
-        menu = gtk.Menu()
         engines = self.__bus.list_active_engines()
-
-        tmp = {}
-        for engine in engines:
-            lang = ibus.get_language_name(engine.language)
-            if lang not in tmp:
-                tmp[lang] = []
-            tmp[lang].append(engine)
-
-        langs = tmp.keys()
-        other = tmp.get(_("Other"), [])
-        if _("Other") in tmp:
-            langs.remove(_("Other"))
-            langs.append(_("Other"))
+        current_engine = \
+            (self.__focus_ic != None and self.__focus_ic.get_engine()) or \
+            (engines and engines[0]) or \
+            None
 
         size = gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)
-        for lang in langs:
-            if len(tmp[lang]) == 1:
-                engine = tmp[lang][0]
-                item = gtk.ImageMenuItem("%s - %s" % (lang, engine.longname))
-                if engine.icon:
-                    item.set_image(_icon.IconWidget(engine.icon, size[0]))
-                else:
-                    item.set_image(_icon.IconWidget(ICON_ENGINE, size[0]))
-                item.connect("activate", self.__im_menu_item_activate_cb, engine)
-                menu.add(item)
+        menu = gtk.Menu()
+        for i, engine in enumerate(engines):
+            lang = ibus.get_language_name(engine.language)
+            item = gtk.ImageMenuItem("%s - %s" % (lang, engine.longname))
+            if current_engine and current_engine.name == engine.name:
+                for widget in item.get_children():
+                    if isinstance(widget, gtk.Label):
+                        widget.set_markup("<b>%s</b>" % widget.get_text())
+            if engine.icon:
+                item.set_image(_icon.IconWidget(engine.icon, size[0]))
             else:
-                item = gtk.MenuItem(lang)
-                menu.add(item)
-                submenu = gtk.Menu()
-                item.set_submenu(submenu)
-                for engine in tmp[lang]:
-                    item = gtk.ImageMenuItem(engine.longname)
-                    if engine.icon:
-                        item.set_image(_icon.IconWidget(engine.icon, size[0]))
-                    else:
-                        item.set_image(_icon.IconWidget(ICON_ENGINE, size[0]))
-                    item.connect("activate", self.__im_menu_item_activate_cb, engine)
-                    submenu.add(item)
+                item.set_image(_icon.IconWidget(ICON_ENGINE, size[0]))
+            item.connect("activate", self.__im_menu_item_activate_cb, engine)
+            menu.add(item)
 
         item = gtk.ImageMenuItem(_("Turn off input method"))
         item.set_image(_icon.IconWidget("gtk-close", size[0]))
         item.connect("activate", self.__im_menu_item_activate_cb, None)
+        if self.__focus_ic == None or not self.__focus_ic.is_enabled():
+            item.set_sensitive(False)
         menu.add(item)
 
         menu.show_all()
@@ -431,6 +459,7 @@ class Panel(ibus.PanelBase):
             menu.show_all()
         else:
             menu = self.__create_im_menu()
+            self.__language_bar.create_im_menu(menu)
         menu.popup(None, None,
                 gtk.status_icon_position_menu,
                 0,
@@ -463,27 +492,30 @@ class Panel(ibus.PanelBase):
             about_dialog.set_icon_name("ibus")
             about_dialog.run()
             about_dialog.destroy()
-        elif command == gtk.STOCK_QUIT: 
+        elif command == gtk.STOCK_QUIT:
             self.__bus.exit(False)
         elif command == "Restart":
             self.__bus.exit(True)
         else:
             print >> sys.stderr, "Unknown command %s" % command
-    
-    def __sigchld_cb(self, sig, sf):
-        try:
-            pid, status = os.wait()
-            if self.__setup_pid == pid:
-                self.__setup_pid = 0
-        except:
-            pass
+
+    def __child_watch_cb(self, pid, status):
+        if self.__setup_pid == pid:
+            self.__setup_pid.close()
+            self.__setup_pid = None
 
     def __start_setup(self):
-        if self.__setup_pid != 0:
-            pid, state = os.waitpid(self.__setup_pid, os.P_NOWAIT)
-            if pid != self.__setup_pid:
+        if self.__setup_pid != None:
+            try:
+                # if setup dialog is running, bring the dialog to front by SIGUSR1
                 os.kill(self.__setup_pid, signal.SIGUSR1)
                 return
-            self.__setup_pid = 0
-        self.__setup_pid = os.spawnl(os.P_NOWAIT, self.__setup_cmd, "ibus-setup")
+            except OSError:
+                # seems the setup dialog is not running anymore
+                self.__setup_pid.close()
+                self.__setup_pid = None
 
+        pid = glib.spawn_async(argv=[self.__setup_cmd, "ibus-setup"],
+                               flags=glib.SPAWN_DO_NOT_REAP_CHILD)[0]
+        self.__setup_pid = pid
+        glib.child_watch_add(self.__setup_pid, self.__child_watch_cb)
